/*  This file is part of the KDE project
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) version 3.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include "audiooutput.h"

namespace Phonon
{
namespace Proxy
{

AudioOutput::AudioOutput(QObject *parent)
    : QObject(parent)
{
    createProxiedObject(BackendInterface::AudioOutputClass);
}

AudioOutput::~AudioOutput()
{
    send(Destructor);
    receive(Destructor);
}

qreal AudioOutput::volume() const
{
    send(AudioOutputVolume);
    qreal r;
    receive(AudioOutputVolume) >> r;
    return r;
}

int AudioOutput::outputDevice() const
{
    send(AudioOutputOutputDevice);
    int r;
    receive(AudioOutputOutputDevice) >> r;
    return r;
}

void AudioOutput::setVolume(qreal newVolume)
{
    send(AudioOutputSetVolume) << newVolume;
    receive(AudioOutputSetVolume);
}

bool AudioOutput::setOutputDevice(int newDevice)
{
    send(AudioOutputSetOutputDevice) << newDevice;
    bool r;
    receive(AudioOutputSetOutputDevice) >> r;
    return r;
}

void AudioOutput::handleSignal(SignalId sig, QDataStream &stream) const
{
    switch (sig) {
    case AudioOutputVolumeChanged:
        {
            qreal v;
            stream >> v;
            emit volumeChanged(v);
        }
        break;
    case AudioOutputAudioDeviceFailed:
        emit audioDeviceFailed();
        break;
    default:
        ProxyObject::handleSignal(sig, stream);
        break;
    }
}

}} //namespace Phonon::Proxy
